/**----------------------------------------------------------------------**/
/** Programm: sw_macro.sas                                               **/
/**    Project    : sas-twophase-package                                 **/
/**    Author     : Schill                                               **/
/**    Date       : 2013-03-13                                           **/
/**    Update     : 2013-08-03    Implem. of maxiter criterion (FitL)    **/
/**                                                                      **/
/**    Context    : sas-twophase-package                                 **/
/**    Purpose    : Parameter estimation                                 **/
/**    Method     : Scott/wild profile likelihood via pseudo model       **/
/**    Procs      : proc logistic, IML                                   **/
/**    Include    : -                                                    **/
/**    Input      : &dat_ein                                             **/
/**    Output     : &dat_aus                                             **/
/**                                                                      **/
/**    Validation :                                                      **/
/**    Status     :                                                      **/
/**    Document.  : path\Docs\ProfileLik.pdf                             **/
/**               : documentation\TPMethods.pdf                          **/
/**    Reference  : Scott/Wild (1997) Fitting Regression Models to Case- **/
/**               : Control Data by Maximum Likelihood. Biometrika 84    **/
/**----------------------------------------------------------------------**/

%MACRO SW_MACRO(
                DAT_EIN =prep,
                DAT_AUS =ml_sw_est,
                MAXITER =10,
                EPSILON =1E-10,
                PROSPS1 =0,
                REGRESS =altstr2 altstr3 altstr4);

 ***************************************************************************;
 * DAT_EIN : Name of prepared input dataset;
 * DAT_AUS : Name of result dataset;
 ***************************************************************************;
 * MAXITER : Max number of iterations                       ;
 * EPSILON : Accuracy of parameter estimation;
 ***************************************************************************;
 * PROSPS1 : Sample1 prospective? (0=no, 1=yes);
 * REGRESS : Regression variables;
 ***************************************************************************;


 %LET PRG_NAME=SW_MACRO.SAS;
 %LET DATETIME_START=%sysfunc(TIME());
 %GLOBAL N_STRAT;

 %MACRO SW_LAUF;

   %BC_PREPS;
   %FitL;
   %SW_VAR;
   run;
 %MEND SW_LAUF;


 *******************************************************************;


 %MACRO BC_PREPS;

   /* read prepared data, build offset variable (prospective and retrospective sampling,
      Intercept and sums for event/trial-syntax*/

   %LET N_STRAT=.;
   data sw_temp1;
        set  &DAT_EIN end=eof;
        if _SAMPLE=1 then do;
           _offset=(1-&PROSPS1)*log(_N1/_N0);
           _anz =_NZ0 + _NZ1;
           _alpha=0;
        end;
        if _SAMPLE=2 then do;
           _offset=log(_M1/_M0);
           _alpha=1;
           _anz =_MZ0 + _MZ1;
        end;
        if eof then call symput("N_STRAT",compress(left(put(_NSTRAT,8.0))));
   run;

title'Prepared data';
*proc print data=sw_temp1;
run;

   %MACRO MODEL1;
      %DO i=1 %TO &N_STRAT; _S&i %END;
   %MEND;

   %MACRO MODEL2;
      _alpha  &REGRESS
   %MEND;

   %* Phase 1 data only;
   proc logistic noprint data=sw_temp1(where=(_SAMPLE=1))
        outest=sw_temp2 (keep=%model1);
        model _NZ1/_anz=%model1 / noint offset=_offset
        ridging=absolute maxiter=&MAXITER absfconv=&EPSILON;
   run;

   %* Build new offsets for phase 2 regression;
   data _null_;set sw_temp2;
        %do i=1 %to &n_strat;
            call symput("par_&i",_S&i);
        %end;
   run;

   data sw_temp1;
      set sw_temp1(where=(_SAMPLE=2));
      by _S;
        keep y _n0 _n1 m_com m_p _alpha &regress nz1 nz2 _offset _S _S1-_S&N_STRAT;
        m_p=_MZ0 + _MZ1;
        %do i=1 %to &n_strat;
          _offset=_offset - &&par_&i*_S&i;
        %end;
        do Y=0 to 1;
           if Y=0 then do;
              m_com=_MZ0;
              nz1  =_NZ0;
              nz2  =_M0;
              output;
           end;
           if Y=1 then do;
              m_com=_MZ1;
              nz1  =_NZ1;
              nz2  =_M1;
              output;
           end;
        end;
   proc sort;
        by Y _S;
   run;

   %* Logistic regression phase 2 (Breslow-Cain);
   proc logistic  noprint data=sw_temp1 desc
        outest=sw_temp3 (keep=%model2);
        model Y  =%model2 / noint offset=_offset
              ridging=absolute maxiter=&MAXITER absfconv=&EPSILON;
        output out=sw_temp4 pred=prob;
        weight m_com;
   run;

   %* Show;
   *proc print data=sw_temp4;
   run;
 %MEND BC_PREPS;


 %Macro  FitL;
   %let ind=0;     /* &ind=1 indicates convergence */
   %let iter=0;    /* counts iterations */

   %do  %until(&ind =1);
     /* sw_temp3 has parameter estimates of previous regression */
     /* sw_temp4 has output dataset of previous regression      */

     %let iter=%eval(&iter +1);
     %* %put 'ITERATION' &iter;
     proc transpose data=sw_temp3 out=sw_tempbeta1;
     run;

     %* Compute stratum-specific offsets;
     data sw_tempg;set sw_temp4;
       by Y _S;
       g= m_p*(y*prob + (1-y)*(1-prob));
     run;
     proc means data=sw_tempg noprint sum;
          by Y _S;
          var g;
          output out=sw_temp5(drop= _type_ _freq_) sum=sumg;
     run;
     data sw_temp6 sw_temp7;merge sw_temp4(keep=Y _S nz1 nz2) sw_temp5;
          by Y _S;
          gamma=nz2-sumg;
          kappa=(nz2-gamma)/(nz1-gamma);
          if Y=0 then output sw_temp6;
          else        output sw_temp7;
     run;
     data sw_temp6;
          merge sw_temp6(keep= _S kappa gamma rename=(kappa=kappa0 gamma=gamma0))
                sw_temp7(keep= _S kappa gamma rename=(kappa=kappa1 gamma=gamma1));
          by _S;
       _offset=log(kappa1/kappa0);
       do y=0 to 1;
            if y=0 then gamma=gamma0;
            if y=1 then gamma=gamma1;
            output;
       end;
     run;
     proc sort;
          by Y _S;
     run;
     data sw_temp1;merge sw_temp1 sw_temp6(keep=Y _s _offset gamma);
          by Y _S;
     run;

     %* logistic regression phase 2;
     proc logistic noprint data=sw_temp1 desc
        outest=sw_temp3 (keep=%model2);
        model Y  =%model2 / noint offset=_offset
              ridging=absolute maxiter=&MAXITER absfconv=&EPSILON;
        output out=sw_temp4 pred=prob;
        weight m_com;
     run;
     proc transpose data=sw_temp3 out=sw_tempbeta(rename=(Col1=beta));
     run;
     data sw_temp6;merge sw_tempbeta1 sw_tempbeta;
          diff=abs(col1-beta);
     run;
     proc means data=sw_temp6 sum noprint;
          var diff;
          output out=sw_temp7(drop= _type_ _freq_) sum=sumdiff;
     run;
     data _null_;set sw_temp7;
       if sumdiff< &epsilon or &iter > &maxiter then
          call symput('ind',1);
     run;
     *proc print data=sw_temp6;
     title"Iteration &iter";
     run;
   %end;

   data sw_temp8;
        conv=1;
        if &iter > &maxiter then conv=0;
   title;
   run;
 %mend Fitl;

 %* Variance-covariance matrix;
 %MACRO SW_VAR;
   proc iml;
    start input;
      use sw_temp4;
          read all var {_N0}      into n0;
          read all var {_N1}      into n1;
          read all var {nz1}      into nz1;       /* Stratum sizes phase 1 */
          read all var {nz2}      into nz2;       /* Stratum sizes phase 2 */
          read all var {m_com}    into m_com;     /* n_ijk */
          read all var {m_p}      into m_p;       /* Cases + controls per pattern (n_0jk + n_1jk) */
          read all var {_S}       into z  ;
          read all var {&REGRESS} into vars;
          read all var {Y}        into y ;
          read all var {prob}   into prob;        /* Probabilities of Breslow-Cain fit */
          read all var {gamma}  into gamma;
          n0=n0[1];
          n1=n1[1];
      close sw_temp4;

      use sw_temp3;
          read all into beta;
      close sw_temp3;
      beta=t(beta);

      %* parm  - vector with parameter-names;
      parm = {%MODEL2};
      %* Designmatrix L for logistic model;
      L= repeat(1,nrow(y),1)||vars;
    finish input;

    start avar;
      var =prob#(1-prob);
      Hess= t(L)*Diag(m_com#var)*L;
      %*Conservative varaince estimate;
      *COV=inv(Hess);

      do j=1 to &N_STRAT;
        %* Computation of k_j= 1/a_j - w_j;
        np   = sum((z=j))/2;                    /* # covariate patterns in stratum j */
        nz20 = sum((Y=0)#(z=j)#m_com);          /* n_0j */
        nz21 = sum((Y=1)#(z=j)#m_com);          /* n_1j */
        nz10 = sum((Y=0)#(z=j)#nz1)/np;         /* N_0j */
        nz11 = sum((Y=1)#(z=j)#nz1)/np;         /* N_1j */
        gam0 = sum((y=0)#(z=j)#gamma)/np;       /* gamma_0j */
        gam1 = sum((y=1)#(z=j)#gamma)/np;       /* gamma_1j */
        a    = 1/(nz20-gam0) + 1/(nz21-gam1) - 1/(nz10-gam0) - 1/(nz11-gam1);
        w    = sum((z=j)#(y=0)#m_p#var);

        if a ^= 0 then do;
           k    = 1/a - w;
           BT   = (z=j)#(y=0)#m_p#var#L;
           BT   = BT[+,];
           Hess = Hess + t(BT)*BT/k;  /* Information, incremented per tratum */
        end;
       *print j nz20 gam0 nz21 gam1 nz10 nz11 a w k;
      end;
      COV=inv(Hess);

      %* Modification of intercept due to retrospective phase one sample;
      if &prosps1=0 then do;
         beta[1] =beta[1] - log(n1/n0);
         COV[1,1]=COV[1,1] - (1/n0 + 1/n1);
      end;
    finish avar;

    run input;
    run avar;

    %* Ausgabe in Dateien;
    parm  = {%MODEL2};
    b     = beta;
    COVb  =COV[1:nrow(beta),1:nrow(beta)];
    stderr=sqrt(vecdiag(COV));
    stderrb=stderr[1:nrow(beta)];
    create sw_temp1 var{parm b stderrb};
      append;
    close sw_temp1;
    create sw_temp2 from COVb[colname=parm];
      append from COVb;
    close sw_temp2;

    * Ausgabedateien zusammenfgen;
    %LET FLAG=1;
    %LET ELAPS=%sysfunc(putn(%sysevalf(%sysfunc(TIME())-&DATETIME_START.),E8601TM12.2)) (hh:mm:ss.ff) ;

    data  &DAT_AUS(label="Prgm: &PRG_NAME, &sysdate"
                             drop =_day _mon _year _hour _min _sec);
    merge sw_temp1(rename=(parm=_parms b=_estim stderrb=_stderr))
          sw_temp2;
     if symget("FLAG") = 1 then do;
        call symput("FLAG","0");
        _program = "&PRG_NAME";
        _source  = "&path_ph1 (Phase 1), &path_ph2 (Phase 2)";
        _regress = "&REGRESS";
        _day     = compress(day(   date()));
        _mon     = compress(month( date()));
        _year    = compress(year(  date()));
        _hour    = compress(hour(  time()));
        _min     = compress(minute(time()));
        _sec     = compress(second(time()));
        if _day < 10 then _day =compress("0"||_day );
        if _mon < 10 then _mon =compress("0"||_mon );
        if _hour< 10 then _hour=compress("0"||_hour);
        if _min < 10 then _min =compress("0"||_min );
        if _sec < 10 then _sec =compress("0"||_sec );
        _time    = compress(_year || "." || _mon || "." || _day) || " " ||
                   compress(_hour || ":" || _min || ":" || _sec);
        _method  = "ml_sw";
        _elaps   = "&ELAPS";
      end;
      label
        _parms   = 'name of parameter'
        _estim   = 'estimate'
        _stderr  = 'standard error'
        _source  = 'source file used by _program'
        _regress = 'regressors used by _program'
        _program = 'program that created this file'
        _time    = 'time of creation of this file'
        _method  = 'method used by _program'
        _elaps   = 'time used by program'
         ;

    %* Issue a message if convergence criterion was not met;
    %let conv=1;
    data _null_;set sw_temp8;
         if conv=0 then
         call symput('conv',conv);
    run;
    %if &conv ne 1 %then %do;
        %put   '+++++++';
        %put   "+ WARNING: Scott/Wild algorithm did not meet convergence criterion (epsil= &epsilon)";
        %put   "+ Probably more than maxit= &maxiter iterations are needed";
        %put   '+++++++';
        proc iml;
           print '+++++++',
                 "+ WARNING: Scott/Wild algorithm did not meet convergence criterion (epsil= &epsilon)",
                 "+ Probably more than maxit= &maxiter iterations are needed",
                 '+++++++';
        quit;
    %end;

    run;

 %MEND SW_VAR;

 %SW_LAUF;

    proc datasets library=work nolist;
         delete sw_temp1 sw_temp2 sw_temp3 sw_temp4 sw_temp5 sw_temp6 sw_temp7 sw_temp8
                sw_tempg sw_tempbeta sw_tempbeta1;
   quit;
%MEND SW_MACRO;
