/*******************************************************************************
*                                                                              *
** PROGRAM NAME: ts_compa.sas                                                  *
** AUTHOR: D. Enders                                                           *
** VERSION/DATE: V 1.0 / 31.10.2011                                            *
*                                                                              *
** PURPOSE: Comparison of different Estimation procedures in two-stage studies *
*                                                                              *
** PROGRAM SPECIFICATION: Subroutine in Macro twophase.sas                     *
*                         returns estimates and standard errors for each       *
*                         estimation procedure in twophase.sas                 *
*                                                                              *
** -READ: &DAT_ZLR/_BC/_WLR/_ML/_S2       (specified in twophase.sas)          *
*                                                                              *
** -WRITE: -                                                                   *                                    *
*                                                                              *
** MODIFICATION NOTES:                                                         *
*                V1.0 - V2.0: NA                                               *
** VALIDATION PROCESS:                                                         *
*                Second program review by W Schill                             *
*                                                                              *
** REFERENCE FOR DOCUMENTATION: -                                              *
*                                                                              *
********************************************************************************/

%MACRO TS_COMPA(
                DAT_pl_sch = zlr_erg,
                DAT_pl_bc  = bc__erg,
                DAT_wl     = wl_erg,
                DAT_ml_em  = ml_em_erg,
                DAT_ml_sw  = ml_sw_erg,
                DAT_s2     = s2__erg,
                HILF_AUS=1);

 options nocenter;

 ***************************************************************************;
 * DAT_PL_SCH : Name of Result dataset  Schill-Method;
 * DAT_PL_BC  : Name of Result dataset  Breslow-Cain-Method;
 * DAT_ML_EM  : Name of Result dataset  EM-Method;
 * DAT_ML_SW  : Name of Result dataset  Profile Likelihood method (Scott/Wild);
 * DAT_WL     : Name of Result dataset  Weighted Regression;
 * DAT_S2     : Name of Result dataset  Sample2-Analysis;
 * HILF_AUS   : Ausgabe der Hilfsparameter _S1 .. _Sn (0=no);
 ***************************************************************************;

 %LET PRG_NAME=TS_COMPA.SAS;
 %LET VERSION  = 16.08.2012;
 %LET TEXT=0;
         %LET MET_NAME=%STR(&methods);
     %DO  count=1 %TO &meth_count;
        %IF %SCAN(&methods,&count)=pl_sch  %THEN %LET MET_NAM&count=%STR("PL (Schill)          ");;
        %IF %SCAN(&methods,&count)=pl_bc   %THEN %LET MET_NAM&count=%STR("PL (Breslow-Cain)    ");;
        %IF %SCAN(&methods,&count)=wl      %THEN %LET MET_NAM&count=%STR("Weighted Regression  ");;
        %IF %SCAN(&methods,&count)=ml_em   %THEN %LET MET_NAM&count=%STR("ML (EM-Algorithm)    ");;
        %IF %SCAN(&methods,&count)=ml_sw   %THEN %LET MET_NAM&count=%STR("ML (Profile Lik.)    ");;
        %IF %SCAN(&methods,&count)=s2      %THEN %LET MET_NAM&count=%STR("Sample2-Analysis     ");;
  %end;

 %* Read Info from datasets;
 %DO M=1 %TO &meth_count;
   %LET ENDUNG=%SCAN(&methods,&m);
   %LET VARS&M=;
   proc sort data=&&DAT_&ENDUNG
             out =ts_compa_temp&M;
     by _parms;
   data ts_compa_temp&M(keep=_parms _estim&M _stderr&M);
   set  ts_compa_temp&M(rename=(_estim=_estim&M _stderr=_stderr&M));
     if _time ne "" then do;
       call symput("_PRG&M",_program);
       call symput("_SOU&M"  ,_source);
       call symput("_REG&M",_regress);
       call symput("_TIM&M",_time);
       call symput("VARS&M",symget("VARS&M")|| " " || compress(_parms));
     end; else do;
       call symput("VARS&M",symget("VARS&M")|| " " || compress(_parms));
     end;
   run;

   %* Hilfsparameter ggf. eliminieren & alphabetische Parameteraufzhlung erstellen;
   %LET NR=0;
   %DO %WHILE(%SCAN(&&_REG&M,&NR+1)^=);
     %LET NR=%EVAL(&NR+1);
   %END;
   %LET _REG_AZ&M=;
   data ts_compa_temp&M;
   set  ts_compa_temp&M;
     if _parms eq "_ALPHA"
       %DO R=1 %TO &NR;
         or _parms eq upcase("%SCAN(&&_REG&M,&R)")
       %END;
    %IF &HILF_AUS eq 0 %THEN %DO; ; %END; %ELSE then;
     call symput("_REG_AZ&M",symget("_REG_AZ&M")|| " " || compress(_parms));
   run;
 %END;


 %* Aggregate results;
 data  ts_compa_temp1;
 merge %DO M=1 %TO &meth_count; ts_compa_temp&M %END;;
   by _parms;
 run;

 *title1 "Program: &PRG_NAME";
 *title2 "Input files: " %DO M=1 %TO &meth_count;
 *       %LET ENDUNG=%SCAN(&methods,&M);
 *       "&&DAT_&ENDUNG "
 *       %END;;

 proc iml;
   start input;
     use ts_compa_temp1;
       read all var {_parms}    into parms;
       %DO M=1 %TO &meth_count;
         read all var {_estim&M} into estim&M;
         read all var {_stderr&M} into stderr&M;
       %END;
     close ts_compa_temp1;
   finish input;

   %* get max_length of Regression Variables;
   %LET N=0;
   %DO %WHILE(%SCAN(&REGR,&N+1,%STR( ))^=);
     %LET N = %EVAL(&N+1);
     %LET REGR&N = %SCAN(&REGR,&N,%STR( ));
   %END;
   %let max_length=%length(%str(_ALPHA));
   %do i=1 %to &N;
       %let length&i=%length(&&REGR&i);
       %if &&length&i > &max_length %then %let max_length=%length(&&REGR&i);
   %end;
   start output;
         %let HEADLINE=%str( );
         %do i=1 %to &max_length;
             %let HEADLINE=&HEADLINE%str( );
         %end;
         %let HEADLINE1=%str("&HEADLINE ");
         %let HEADLINE2=%str("&HEADLINE ");
         %let ls=%eval(%sysfunc(getoption(LS))-(&max_length+1));
         %DO M=1 %TO &meth_count;
           %if &ls<22 %then %do; %let break=1; %end; %else %do; %let break=0;%end;
           %if &break=0 %then %do;
       result1 = result1 || estim&M || stderr&M;
       col1    = col1    || {"estim" "stderr"};
           %end;
           %else %if &break=1 %then %do;
       result2 = result2 || estim&M || stderr&M;
       col2    = col2    || {"estim" "stderr"};
           %end;
       %LET NR=%eval(&M);
           %if &break=0 %then %do; %LET HEADLINE1=&HEADLINE1 &&MET_NAM&NR; %end;
           %else %if &break=1 %then %do; %let HEADLINE2=&HEADLINE2 &&MET_NAM&NR; %end;
           %let ls=%eval(&ls-22);
     %END;

     reset noname;
     print /&HEADLINE1,
           result1[rowname=parms colname=col1 format=10.5];
         %if %length(&HEADLINE2)>%eval(&max_length+5) %then %do;
     print /&HEADLINE2,
           result2[rowname=parms colname=col2 format=10.5];
     reset name;
         %end;
   finish output;

   run input;
   run output;
   quit;
   proc datasets library=work nolist;
        delete %do m=1 %to &meth_count; ts_compa_temp&m %end;;
   quit;


%MEND TS_COMPA;
