/**---------------------------------------------------------------------**/
/** Programm: zlrmacro.sas                                              **/
/**    Project    : sas-twophase-package                                **/
/**    Author     : Schill, Seiler                                      **/
/**    Date       : 1999-07-27                                          **/
/**    Update     : 2011-11-02  Macroparms DOKU, DEBUG deleted          **/
/**    Update     : 2012-07-31  Replace IML-routine with proc logistic  **/
/**                                                                     **/
/**    Context    : sas-twophase-package                                **/
/**    Purpose    : Parameter estimation                                **/
/**    Method     : Pseudo Likelihood (Schill et al.)                   **/
/**    Procs      : proc logistic, IML                                  **/
/**    Include    : -                                                   **/
/**    Input      : &dat_ein (prepared dataset)                         **/
/**    Output     : &dat_out (result dataset)                           **/
/**                                                                     **/
/**    Validation : Enders                                              **/
/**    Status     :                                                     **/
/**    Doku       : Documentation                                       **/
/**    Reference  : Schill/Drescher (1997). Stats in Medicine           **/
/**                 Schill et al. (1993), Biometrika                    **/
/**                 Schill (1991), PhD thesis                           **/
/**---------------------------------------------------------------------**/

%MACRO ZLRMACRO(
                DAT_EIN =prep,
                DAT_AUS =zlr_est,
                MAXITER =10,
                EPSILON =1E-10,
                PROSPS1 =0,
                REGRESS =altstr2 altstr3 altstr4);

 *************************************************************************;
 * DAT_EIN : Name of prepared input data;
 * DAT_AUS : Name of results dataset;
 *************************************************************************;
 * MAXITER : Max Number of iteration                        ;
 * EPSILON : Accuracy of parameter estimation;
 *************************************************************************;
 * PROSPS1 : Sample1 prospective (0=no, 1=yes);
 * REGRESS : Regression varibles;
 *************************************************************************;

 %LET PRG_NAME=ZLRMACRO.SAS;
 %LET DATETIME_START=%sysfunc(TIME());
 %GLOBAL N_STRAT VARS;

 %MACRO ZLR_LAUF;
   %ZLRPREPS;
   %ZLR_PCLS;
 %MEND ZLR_LAUF;


 %MACRO ZLRPREPS;

   %LET N_STRAT=.;
   data _NULL_;
     set &DAT_EIN end=eof;
     if eof then call symput("N_STRAT",compress(left(put(_S,8.0))));
   run;


   %MACRO MODEL;
      %DO i=1 %TO &N_STRAT; _S&i %END;
           _alpha &REGRESS
   %MEND;

   %LET VARS=%STR(%MODEL);
   %LET NP=0;
   %DO %WHILE(%SCAN(&VARS,&NP+1)^=);
     %LET NP=%EVAL(&NP+1);
     %LET VAR&NP=%SCAN(&VARS,&NP);
   %END;

   data sch_temp1;
     set &DAT_EIN;
     if _SAMPLE=1 then do;
       * _alpha and regressor =0;
       %DO i= %EVAL(&N_STRAT + 1) %TO &NP; &&VAR&i=0; %END;
       _offset=(1-&PROSPS1)*log(_N1/_N0);
       _anz1=_NZ1;
       _anz =_NZ0 + _NZ1;
     end;
     if _SAMPLE=2 then do;
       %DO i=1 %TO &N_STRAT; _S&i=-_S&i; %END;
       _offset=log(_M1/_M0);
       _alpha=1;
       _anz1=_MZ1;
       _anz =_MZ0 + _MZ1;
     end;
   run;

   proc sort data=sch_temp1;
     by _SAMPLE _S;
   run;

   %* Parameter estimation;
   proc logistic data=sch_temp1 noprint
        outest=sch_temp2(keep=%model);
        model _anz1/_anz = %model/ offset=_offset noint
              ridging=absolute maxiter=&maxiter absfconv=&epsilon;
   run;
 %MEND ZLRPREPS;


 %MACRO ZLR_PCLS;

   %LET NP=0;
   %DO %WHILE(%SCAN(&VARS,&NP+1)^=);
     %LET NP=%EVAL(&NP+1);
     %LET VAR&NP=%SCAN(&VARS,&NP);
   %END;

   proc iml;
    start input;
      use sch_temp1;
        read all var {_NZ0}    into wgt10  where(_SAMPLE=1);
        read all var {_NZ1}    into wgt11  where(_SAMPLE=1);
        read all var {_MZ0}    into wgt20  where(_SAMPLE=2);
        read all var {_MZ1}    into wgt21  where(_SAMPLE=2);
        read all var {&vars}   into x1     where(_SAMPLE=1);
        read all var {&vars}   into x2     where(_SAMPLE=2);
        read all var {_offset} into offs1  where(_SAMPLE=1);
        read all var {_offset} into offs2  where(_SAMPLE=2);
      close sch_temp1;

      *dimensions;
      nz  =nrow(wgt10);
      nx  =nrow(wgt20);
      *responses, counts and regressors;
      y1     = repeat(1,nz,1)//repeat(0,nz,1);
      y2     = repeat(1,nx,1)//repeat(0,nx,1);
      y      = y1//y2;
      wgt    = wgt11//wgt10//wgt21//wgt20;
      n      = sum(wgt11+wgt10);
      x      = x1//x1//x2//x2;
      offset = offs1//offs1//offs2//offs2;
      * numbers in Sample2 per stratum;
      _X2 = -t(x2[,1:&N_STRAT]);
      m20 = (_X2*wgt20);
      m21 = (_X2*wgt21);

      * z Distribution in Sample1;
      _X1 = t(x1[,1:&N_STRAT]);

      *parm  - vector with parameter-names;
      parm = { %do i=1 %to &np;
                   &&var&i
                   %if &i < &np %then ,;
               %end;};
    finish input;

    start varpcls;
    %* PCL-Variance 2-phase logistic regression (Schill);

      %* read estimates;
      use sch_temp2;
      read all into b;
      close sch_temp2;
      b=t(b);
      eta= offset + x*b;
      p=1/(1+exp(-eta));
      f=p#(1-p);
      xpxi = inv(x`*(wgt#f#x));

      %let p =%eval(&np - &n_strat);
      *prospective Covariance;
      COV_P = xpxi;
      COR_P = inv(sqrt(diag(COV_P)))*COV_P*inv(sqrt(diag(COV_P)));
      H = (X`*(wgt#f#X))/n;
      Hi= inv(H);

      *Modification due to 2-phase setup
      *  (Diss, p.29, 31) ;
      nrx1= nrow(x1);
      f1  = f[1:nrx1] # (wgt11+wgt10);
      V   = diag((_X1*f1)/n);
      TA  = H[1:&n_strat,1:&n_strat] - V;
      TB  = H[1:&n_strat,&n_strat+1:&np];
      Q   = n*(inv(diag(m20)) + inv(diag(m21)));

      Ojp=J(&n_strat,&p,0);
      Opj=t(Ojp);
      Opp=J(&p,&p,0);
      UP = TA||TB;
      LO = Opj||Opp;
      RI = Up//LO;
      rho= n/sum(wgt10)+n/sum(wgt11);
      VVT= V[,+]@V[+,];
      SS1= rho*VVT;

      DS2= t(RI)*((Q||Ojp)//LO)*RI;
      DS1= (1-&prosps1)*(ss1||Ojp//LO);
      COV= inv(n)*Hi*(H - DS1 - DS2)*Hi ;
      stderr = sqrt(vecdiag(COV));
    finish varpcls;


    run input;
    run varpcls;

    * Output to result dataset;
    create sch_temp1 var{parm b stderr};
      append;
    close sch_temp1;
    create sch_temp2 from COV[colname=parm];
      append from COV;
    close sch_temp2;

    * Combine output data;
    %LET FLAG=1;
        %LET ELAPS=%sysfunc(putn(%sysevalf(%sysfunc(TIME())-&DATETIME_START.),E8601TM12.2)) (hh:mm:ss.ff) ;
    data  &DAT_AUS(label="Prgm: &PRG_NAME, &sysdate"
                             drop =_day _mon _year _hour _min _sec);
    merge sch_temp1(rename=(parm=_parms b=_estim stderr=_stderr))
          sch_temp2;
      if symget("FLAG") = 1 then do;
        call symput("FLAG","0");
        _program = "&PRG_NAME";
        _source  = "&path_ph1 (Phase 1), &path_ph2 (Phase 2)";
        _regress = "&REGRESS";
        _day     = compress(day(   date()));
        _mon     = compress(month( date()));
        _year    = compress(year(  date()));
        _hour    = compress(hour(  time()));
        _min     = compress(minute(time()));
        _sec     = compress(second(time()));
        if _day < 10 then _day =compress("0"||_day );
        if _mon < 10 then _mon =compress("0"||_mon );
        if _hour< 10 then _hour=compress("0"||_hour);
        if _min < 10 then _min =compress("0"||_min );
        if _sec < 10 then _sec =compress("0"||_sec );
        _time    = compress(_year || "." || _mon || "." || _day) || " " ||
                   compress(_hour || ":" || _min || ":" || _sec);
        _method  = "pl_sch";
        _elaps   = "&ELAPS";
      end;
      label
        _parms   = 'name of parameter'
        _estim   = 'estimate'
        _stderr  = 'standard error'
        _source  = 'source file used by _program'
        _regress = 'regressors used by _program'
        _program = 'program that created this file'
        _time    = 'time of creation of this file'
        _method  = 'method used by _program'
        _elaps   = 'time used by _program'
          ;
      ;
   run;
   proc datasets library=work nolist;
        delete sch_temp1 sch_temp2;
   quit;

 %MEND ZLR_PCLS;

 %ZLR_LAUF;

%MEND ZLRMACRO;
