/**--------------------------------------------------------------------**/
/** Programm: af2p.sas                                                 **/
/**    Project    : adjusted attrib fractions after 2p analysis        **/
/**    Author     : Schill                                             **/
/**    Date       : 2013-12-16                                         **/
/**    Update     : 2014-02-11  Module C_AI                            **/
/**                                                                    **/
/**    Context    : sas-twophase-package                               **/
/**    Purpose    : Compute confounder-adjusted attributable fractions **/
/**                 from two-phase data                                **/
/**    Method     : based on Poisson model via EM algorithm            **/
/**    Procs      : IML                                                **/
/**    Include    : -                                                  **/
/**    Input      : em_design, em_theta, em_final                      **/
/**    Output     : af                                                 **/
/**                                                                    **/
/**    Validation : Results checked by K Drescher                      **/
/**    Status     :                                                    **/
/**    Doku       : AFDocu.pdf                                         **/
/**    Reference  : Greenland/Drescher, Drescher/Osius (1993)          **/
/**                 Schill et al. (2014)                               **/
/**--------------------------------------------------------------------**/
%macro af2p(
       pathdesign=em_design,
       pathparm  =em_theta,
       pathfinal =em_final,
       pathout   =af,
       regress   =,            /* original regressor */
       factor    =,
       nameF     =,
       study     =,
       s1pr      =0);

/*--------------------------------------------------------------------
  &pathdesign  - Design matrix X of Poisson model in underlying two-phase analysis
  &pathparm    - Estimated parameter and Covariance theta, Cov
  &pathfinal   - Estimated expected counts
  &patout      - Result dataset
  &regress     - Terms in linear predictor of underlying analysis
  &factor      - Terms pertaining to factor(s) for which attributable
                 fraction is to be computed
  &nameF       - Name of factor
  &study       - Study characteristics
  &s1pr        - Phase one study prospective ? (1: yes/0: no (default))
                 value must equal that of underlying analysis
--------------------------------------------------------------------*/

   %* Squeeze strings;
   data _null_;
      regress="&regress";
      factor ="&factor";
      call symput('regress',compbl(regress));
      call symput('factor',compbl(factor));
   run;

   %LET NP=0;
   %DO %WHILE(%SCAN(&regress,&NP+1)^=);
     %LET NP=%EVAL(&NP+1);
     %let REG&NP=%UPCASE(%SCAN(&regress,&NP));
   %END;

   %LET NPi=0;
   %DO %WHILE(%SCAN(&factor,&NPi+1)^=);
     %LET NPi=%EVAL(&NPi+1);
     %let Fac&NPi=%UPCASE(%SCAN(&factor,&NPi));
   %END;

   %* Find place of each term in &factor in sequence of terms in &regress;
   %let mm1=0;
   %do i1=1 %to &npi;
   %let place&i1=.;
      %do i=1 %to &np;
          %if &&fac&i1=&&reg&i %then %let place&i1=&i;
      %end;
      %if &&place&i1=. %then %let mm1=%eval(&mm1+1);
   %end;
   run;

   %macro ccheck;
   /*-------------------------------------------------------------------
    The macro checks terms consistency, i.e.,
     (a) the terms i &regress are in the correct sequence and agree with
         the terms in the underlying analysis
     (b) the terms in &factor are a subset of those in &regress
   -------------------------------------------------------------------*/
      %* (a);
      data af_temp1;set &pathparm(keep=_parms);
           if upcase(_parms)='_ALPHA' or substr(upcase(_parms),1,5)='DELTA' then delete;
      run;
      data af_temp1;set af_temp1;
           z+1;
           call symput("parm"||left(z),left(_parms));
           call symput('np_org',trim(z));
      run;
      data _null_;
           %let mm=0;
           %if &np_org ^= &np %then %let mm=%eval(&mm+1);
           %do i=1 %to &np;
               %if %upcase(&&reg&i) ^= %upcase(&&parm&i) %then %let mm=%eval(&mm+1);
           %end;
      run;
      %if &mm ^= 0 %then %do;
         %put '++++++++++++++++++';
         %put 'ERROR: Mismatch of regression variables or improper sequence in &regress';
         %put '++++++++++++++++++';
          proc iml;
             print '++++++++++++++++++',
                   'ERROR: Mismatch of regression variables or improper sequence in &regress',
                   '++++++++++++++++++';
          quit;
      %end;

      %* (b);
      %if &mm1 ^= 0 %then %do;
          %put '++++++++++++++++++';
          %put 'ERROR: Terms in &factor do not match terms in &regress';
          %put '++++++++++++++++++';
          proc iml;
             print '++++++++++++++++++',
                   'ERROR: Terms in &factor do not match terms in &regress',
                   '++++++++++++++++++';
          quit;
      %end;
      run;
   %mend ccheck;
   %ccheck;

    proc iml ;
        start input;
          use &pathdesign;
              %* reads parts of design matrix pertaining to cases and controls;
              read all      into X1 where(_alpha=1);
              read all      into X0 where(_alpha=0);
          close &pathdesign;
          use &pathfinal;
              %* reads mu0 and mu1;
              read all var {mu} into mu0 where(y=0);
              read all var {mu} into mu1 where(y=1);
          close &pathfinal;
          use &pathparm;
              %* reads theta and cov;
              read all into em_theta;
          close &pathparm;
          theta=em_theta[,1];
          cov  =em_theta[,3:ncol(em_theta)];
        finish input;
        run input;

        Regress="&regress";
        Factor ="&factor";
        NameF  ="&nameF";
        Study  ="&study";
        Sysdate="&sysdate";

        nxz=nrow(X1);
        n1=sum(mu1);
        n0=sum(mu0);

        %* Re-correct alpha=theta[1], if necessary;
        %if &s1pr=0 %then %do;
            theta[1]=theta[1] + log(n1/n0);
        %end;

        %* pfactor has the columns where X1 has to be modified;
        pfactor=repeat(0,1,&npi);
        %do i=1 %to &npi;
            pfactor[&i]=&&place&i;
        %end;

        start afrac;

           nv  =repeat(0,nxz,1);     /* null vector */
           X1s =X1;                  /* to yield X1star */
               n_i=&npi;
               /* Select columns where X1s must be zero */
               i_i=PFactor;
              do _n=1 to n_i;
                X1s[,i_i[_n]+1]=nv;
              end;

              mu1s=exp(X1s*theta);
              n1=sum(mu1);
              *print n1  mu1 mu1s;
              AF =1-sum(mu1s)/n1;

            * Poisson Variance of AF (Equations 9 and 10);
              Dmu1      = X1#mu1;
              Smu1      = Dmu1[+,];
              Dmu1s     = X1s#mu1s;
              Smu1s     = Dmu1s[+,];
              %* dAF/dTheta;
              dAF_dT    = - Smu1s/N1 +(1-AF)*Smu1/N1;
              V_AF      = dAF_dT*Cov*t(dAF_dT);
              SE_AF=sqrt(t(V_AF));
              CL_AF=AF - 1.96*SE_AF;
              CU_AF=AF + 1.96*SE_AF;

              start C_AI;
                * Check asymptotic independence (zero covariance) of AF and N_i;
                * t0 and t1 should be zero;
                Dmu0      = X0#mu0;
                Smu0      = Dmu0[+,];
                t0= dAF_dT*Cov*t(Smu0);
                t1= dAF_dT*Cov*t(Smu1);
                print t0 t1;
              finish C_AI;

            print NameF AF SE_AF CL_AF CU_AF;
        finish afrac;

        start AFout;
        * Output into data files;
          create &pathout var{Sysdate NameF AF SE_AF CL_AF CU_AF Factor Regress Study};
            append;
          close &pathout;
        finish AFout;

        run afrac;
        run AFout;
    quit;
    data &pathout(label="Prgm: &prgname, &sysdate");
         length NameF  $ 25;
         length Factor $ %length(&regress);
         set &pathout;
         Label AF     ='Attributable Fraction'
               SE_AF  ='Std. Error'
               CL_AF  ='Lower 95%-CL'
               CU_AF  ='Upper 95%-CL'
               Factor ='Terms Removed'
               ;
    proc datasets library=work nolist;
         delete af_temp1;
    quit;
%mend af2p;
